#######################################################################
#
# NCM component for NFS daemon
#
#                 Managed files:
#                 /etc/exports
#                 /etc/fstab
#
# Charles Loomis <charles.loomis@cern.ch>
# 0.0.7 : 21/12/04 14:35
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::nfs;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;
use File::Copy;

use EDG::WP4::CCM::Element;

use File::Path;


local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # The absolute names of configuration files. 
    my $exports = "/etc/exports";
    my $fstab = "/etc/fstab";

    # Path names.
    my $base = "/software/components/nfs";
    my $exportsPath = "$base/exports";
    my $mountsPath = "$base/mounts";

    # Collect information for exports file.
    my $contents .= "#\n# File generated by ncm-nfs on " . 
                    localtime() . ".\n#\n";
	    
    if ($config->elementExists($exportsPath)) {

	# Pull out the parameters from the configuration. 
	my $exports = $config->getElement($exportsPath);
	
	# Loop through all of the servers creating a line for each.
	while ($exports->hasNextElement()) {
	    
	    my $element = $exports->getNextElement();
	    
	    if ( $element->isResource() ) {
		my %entry = $element->getHash();

		# Pull out the path from the profile.
		my $property = $entry{path};
		unless (defined($property)) {
		    $self->error("undefined path for nfs entry; skipping");
		    next;
		}
		unless ($property->isProperty()) {
		    $self->error("path is not a property; skipping");
		    next;
		}
		my $path = $property->getValue();

		# Pull out the host resource from the profile. 
                my $hosts = $entry{hosts};
		unless (defined($hosts)) {
		    $self->error("undefined hosts for nfs entry; skipping");
		    next;
		}

		# It was defined.  Get the list of hosts. 
                my @list = $hosts->getList();

		# Collect all of the host/option entries.
		my $entries = '';
		foreach (@list) {
		    unless ($_->isProperty()) {
			$self->error("hosts entry not a property; skipping");
			next;
		    }
		    $entries .= $_->getValue() . ' ';
		}

		# Only actually write the line if there was at least one 
		# valid host/option entry. 
		if ($entries ne '') {
		    $contents .= "$path $entries\n";
		}

	    }
	}
    }

    # Now just create the new configuration file.  Be careful to save
    # a backup of the previous file if necessary.
    if ( ! -e $exports ) {
         
        # Configuration file doesn't exist yet.  Create it.
        open ( CONF,">$exports" );
        print CONF $contents;
        close (CONF);
        $self->log("$exports created");
         
    } else {
         
        # Already exists. Make backup and create new file.
        my $result = LC::Check::file( $exports,
                                      backup => ".old",
                                      contents => $contents,
                                      );
        $self->log("$exports updated") if $result;
    }


    # Now move on to /etc/fstab.  First read in all of the existing
    # information.  

    # Collect information for fstab file.
    $contents = "#\n# File generated by ncm-nfs on " . 
	         localtime() . ".\n#\n";
    $contents .= "# Only nfs file systems managed by ncm-nfs component.\n";

    # Extract fstab information.
    my %oldnfs;
    open FSTAB, "<$fstab";
    while (<FSTAB>) {
	unless (m/^\s*$/ or m/^\s*\#/) {
	    my ($device,$mntpt,$fstype,$opt,$freq,$passno) = split;
	    $opt = "defaults" unless $opt;
	    $freq = 0 unless defined($freq);
	    $passno = 0 unless defined($passno);
	    
	    if ($fstype =~ /^nfs/) {
		# It is an nfs entry, save the information.
		$oldnfs{$device} = {"device" => $device,
				    "mntpt" => $mntpt,
				    "fstype" => $fstype,
				    "opt" => $opt,
				    "freq" => $freq,
				    "passno" => $passno};
	    } else {
		# If the fstype doesn't begin with nfs (i.e. nfs or nfs4),
		# then just add the line to the output. 
		$contents .= $_;
	    }
	}
    }
    close FSTAB;

    # Now check the information given in the configuration.  
    my %newnfs;
    if ($config->elementExists($mountsPath)) {

	# Pull out the parameters from the configuration. 
	my $exports = $config->getElement($mountsPath);
	
	# Loop through all of the servers creating a line for each.
	while ($exports->hasNextElement()) {
	    
	    my $element = $exports->getNextElement();
	    
	    if ( $element->isResource() ) {
		my %entry = $element->getHash();

		# Pull out the device from the profile.
		my $property = $entry{device};
		my $device = $property->getValue();

		# Pull out the mountpoint from the profile
		$property = $entry{mountpoint};
		my $mountpoint = $property->getValue();

		# Pull out the fstype from the profile
		$property = $entry{fstype};
		my $fstype = $property->getValue();

		# Pull out the options from the profile.
		$property = $entry{options};
		my $options = 
		    defined($property) ? $property->getValue() : "defaults";

		# Pull out the freq from the profile.
		$property = $entry{freq};
		my $freq = 
		    defined($property) ? $property->getValue() : 0;

		# Pull out the freq from the profile.
		$property = $entry{passno};
		my $passno = 
		    defined($property) ? $property->getValue() : 0;

		# Add the entry. 
		$newnfs{$device} = {"device" => $device,
				    "mntpt" => $mountpoint,
				    "fstype" => $fstype,
				    "opt" => $options,
				    "freq" => $freq,
				    "passno" => $passno,
				    "new" => 1};
	    }
	}
    }

    # See which entries are modified or new.  (I.e. remove from the
    # oldnfs hash ones which appear in newnfs. 

    # Unmount the file systems which are going away. 
    foreach (keys %oldnfs) {
	my $hashref = $oldnfs{$_};
	my %hash = %$hashref;
	my $mntpt = $hash{mntpt};
	$self->log("Unmounting $mntpt\n");
	system("umount -l $mntpt");
	if ($?) {
	    $self->warn("Error unmounting $mntpt\n");
	} else {

	    # Try to remove the mount point.  Issue a warning on
	    # failure. 
	    unless (rmdir $mntpt) {
		$self->warn("can't delete mountpoint $mntpt");
	    }
	}
    }

    # Add to the configuration file the new NFS locations.  Creating
    # the new mount points as necessary.  
    foreach (keys %newnfs) {
	my $hashref = $newnfs{$_};
	my %hash = %$hashref;
	my $mntpt = $hash{mntpt};
	my $new = $hash{new};

	$contents .= 
	    $hash{device} . ' ' . 
	    $hash{mntpt} . ' ' . 
	    $hash{fstype} . ' ' . 
	    $hash{opt} . ' ' . 
	    $hash{freq} . ' ' . 
	    $hash{passno} . "\n";

	# If the directory doesn't exist, then create it.  Issue a
        # warning if this couldn't be done. 
	mkpath($mntpt,0,0755) unless (-e $mntpt);
	unless (-d $mntpt) {
	    $self->warn("error creating mountpoint $mntpt");
	}
    }

    # Now just create the new configuration file.  Be careful to save
    # a backup of the previous file if necessary.
    if ( ! -e $fstab ) {
         
        # Configuration file doesn't exist yet.  Create it.
        open ( CONF,">$fstab" );
        print CONF $contents;
        close (CONF);
        $self->log("$fstab created");
         
    } else {
         
        # Already exists. Make backup and create new file.
        my $result = LC::Check::file( $fstab,
                                      backup => ".old",
                                      contents => $contents,
                                      );
        $self->log("$fstab updated") if $result;
    }

    # Mount or remount as appropriate.
    foreach (keys %newnfs) {
	my $hashref = $newnfs{$_};
	my %hash = %$hashref;
	my $mntpt = $hash{mntpt};
	my $new = $hash{new};
	if ($new) {
	    $self->log("Mounting $mntpt\n");
	    system("mount $mntpt");
	    if ($?) {
		$self->warn("Error mounting $mntpt\n");
	    }
	} else {
	    $self->log("Remounting $mntpt\n");
	    system("mount -o remount $mntpt");
	    if ($?) {
		$self->warn("Error remounting $mntpt\n");
	    }
	}
    }

    return 1;
}

1;      # Required for PERL modules
